function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
import { CategoricalColorNamespace, getNumberFormatter, smartDateVerboseFormatter } from '@superset-ui/core';
import { ForecastSeriesEnum } from '../types';
import { parseYAxisBound } from '../utils/controls';
import { extractTimeseriesSeries } from '../utils/series';
import { extractForecastSeriesContext, extractProphetValuesFromTooltipParams, formatProphetTooltipSeries, rebaseTimeseriesDatum } from '../utils/prophet';
import { defaultGrid, defaultTooltip, defaultYAxis } from '../defaults';
export default function transformProps(chartProps) {
  const {
    width,
    height,
    formData,
    queryData
  } = chartProps;
  const {
    area,
    colorScheme,
    contributionMode,
    forecastEnabled,
    seriesType,
    logAxis,
    opacity,
    stack,
    markerEnabled,
    markerSize,
    minorSplitLine,
    truncateYAxis,
    yAxisFormat,
    yAxisBounds,
    zoomable
  } = formData;
  const colorFn = CategoricalColorNamespace.getScale(colorScheme);
  const rebasedData = rebaseTimeseriesDatum(queryData.data || []);
  const rawSeries = extractTimeseriesSeries(rebasedData);
  const series = [];

  const _formatter = getNumberFormatter(contributionMode ? ',.0%' : yAxisFormat);

  rawSeries.forEach(entry => {
    const forecastSeries = extractForecastSeriesContext(entry.name || '');
    const isConfidenceBand = forecastSeries.type === ForecastSeriesEnum.ForecastLower || forecastSeries.type === ForecastSeriesEnum.ForecastUpper;
    const isObservation = forecastSeries.type === ForecastSeriesEnum.Observation;
    const isTrend = forecastSeries.type === ForecastSeriesEnum.ForecastTrend;
    let stackId;

    if (isConfidenceBand) {
      stackId = forecastSeries.name;
    } else if (stack && isObservation) {
      // the suffix of the observation series is '' (falsy), which disables
      // stacking. Therefore we need to set something that is truthy.
      stackId = 'obs';
    } else if (stack && isTrend) {
      stackId = forecastSeries.type;
    }

    let plotType;

    if (!isConfidenceBand && (seriesType === 'scatter' || forecastEnabled && isObservation)) {
      plotType = 'scatter';
    } else if (isConfidenceBand) {
      plotType = 'line';
    } else {
      plotType = seriesType === 'bar' ? 'bar' : 'line';
    }

    const lineStyle = isConfidenceBand ? {
      opacity: 0
    } : {};
    if (!((stack || area) && isConfidenceBand)) series.push(_extends({}, entry, {
      id: entry.name,
      name: forecastSeries.name,
      itemStyle: {
        color: colorFn(forecastSeries.name)
      },
      type: plotType,
      // @ts-ignore
      smooth: seriesType === 'smooth',
      step: ['start', 'middle', 'end'].includes(seriesType) ? seriesType : undefined,
      stack: stackId,
      lineStyle,
      areaStyle: {
        opacity: forecastSeries.type === ForecastSeriesEnum.ForecastUpper || area ? opacity : 0
      },
      symbolSize: !isConfidenceBand && (plotType === 'scatter' || forecastEnabled && isObservation || markerEnabled) ? markerSize : 0
    }));
  }); // yAxisBounds need to be parsed to replace incompatible values with undefined

  let [min, max] = (yAxisBounds || []).map(parseYAxisBound); // default to 0-100% range when doing row-level contribution chart

  if (contributionMode === 'row' && stack) {
    if (min === undefined) min = 0;
    if (max === undefined) max = 1;
  }

  const echartOptions = {
    grid: _extends({}, defaultGrid, {
      top: 30,
      bottom: zoomable ? 80 : 0,
      left: 20,
      right: 20
    }),
    xAxis: {
      type: 'time'
    },
    yAxis: _extends({}, defaultYAxis, {
      type: logAxis ? 'log' : 'value',
      min,
      max,
      minorTick: {
        show: true
      },
      minorSplitLine: {
        show: minorSplitLine
      },
      axisLabel: {
        formatter: _formatter
      },
      scale: truncateYAxis
    }),
    tooltip: _extends({}, defaultTooltip, {
      trigger: 'axis',
      formatter: params => {
        // @ts-ignore
        const rows = ["" + smartDateVerboseFormatter(params[0].value[0])]; // @ts-ignore

        const prophetValues = extractProphetValuesFromTooltipParams(params);
        Object.keys(prophetValues).forEach(key => {
          const value = prophetValues[key];
          rows.push(formatProphetTooltipSeries(_extends({}, value, {
            seriesName: key,
            formatter: _formatter
          })));
        });
        return rows.join('<br />');
      }
    }),
    legend: {
      data: rawSeries.filter(entry => extractForecastSeriesContext(entry.name || '').type === ForecastSeriesEnum.Observation).map(entry => entry.name || ''),
      right: zoomable ? 80 : 'auto'
    },
    series,
    toolbox: {
      show: zoomable,
      feature: {
        dataZoom: {
          yAxisIndex: false,
          title: {
            zoom: 'zoom area',
            back: 'restore zoom'
          }
        }
      }
    },
    dataZoom: zoomable ? [{
      type: 'slider',
      start: 0,
      end: 100,
      bottom: 20
    }] : []
  };
  return {
    area,
    colorScheme,
    contributionMode,
    // @ts-ignore
    echartOptions,
    seriesType,
    logAxis,
    opacity,
    stack,
    markerEnabled,
    markerSize,
    minorSplitLine,
    width,
    height
  };
}