"use strict";

exports.__esModule = true;
exports.default = transformProps;

var _core = require("@superset-ui/core");

var _types = require("../types");

var _controls = require("../utils/controls");

var _series = require("../utils/series");

var _prophet = require("../utils/prophet");

var _defaults = require("../defaults");

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function transformProps(chartProps) {
  const {
    width,
    height,
    formData,
    queryData
  } = chartProps;
  const {
    area,
    colorScheme,
    contributionMode,
    forecastEnabled,
    seriesType,
    logAxis,
    opacity,
    stack,
    markerEnabled,
    markerSize,
    minorSplitLine,
    truncateYAxis,
    yAxisFormat,
    yAxisBounds,
    zoomable
  } = formData;

  const colorFn = _core.CategoricalColorNamespace.getScale(colorScheme);

  const rebasedData = (0, _prophet.rebaseTimeseriesDatum)(queryData.data || []);
  const rawSeries = (0, _series.extractTimeseriesSeries)(rebasedData);
  const series = [];

  const _formatter = (0, _core.getNumberFormatter)(contributionMode ? ',.0%' : yAxisFormat);

  rawSeries.forEach(entry => {
    const forecastSeries = (0, _prophet.extractForecastSeriesContext)(entry.name || '');
    const isConfidenceBand = forecastSeries.type === _types.ForecastSeriesEnum.ForecastLower || forecastSeries.type === _types.ForecastSeriesEnum.ForecastUpper;
    const isObservation = forecastSeries.type === _types.ForecastSeriesEnum.Observation;
    const isTrend = forecastSeries.type === _types.ForecastSeriesEnum.ForecastTrend;
    let stackId;

    if (isConfidenceBand) {
      stackId = forecastSeries.name;
    } else if (stack && isObservation) {
      // the suffix of the observation series is '' (falsy), which disables
      // stacking. Therefore we need to set something that is truthy.
      stackId = 'obs';
    } else if (stack && isTrend) {
      stackId = forecastSeries.type;
    }

    let plotType;

    if (!isConfidenceBand && (seriesType === 'scatter' || forecastEnabled && isObservation)) {
      plotType = 'scatter';
    } else if (isConfidenceBand) {
      plotType = 'line';
    } else {
      plotType = seriesType === 'bar' ? 'bar' : 'line';
    }

    const lineStyle = isConfidenceBand ? {
      opacity: 0
    } : {};
    if (!((stack || area) && isConfidenceBand)) series.push(_extends({}, entry, {
      id: entry.name,
      name: forecastSeries.name,
      itemStyle: {
        color: colorFn(forecastSeries.name)
      },
      type: plotType,
      // @ts-ignore
      smooth: seriesType === 'smooth',
      step: ['start', 'middle', 'end'].includes(seriesType) ? seriesType : undefined,
      stack: stackId,
      lineStyle,
      areaStyle: {
        opacity: forecastSeries.type === _types.ForecastSeriesEnum.ForecastUpper || area ? opacity : 0
      },
      symbolSize: !isConfidenceBand && (plotType === 'scatter' || forecastEnabled && isObservation || markerEnabled) ? markerSize : 0
    }));
  }); // yAxisBounds need to be parsed to replace incompatible values with undefined

  let [min, max] = (yAxisBounds || []).map(_controls.parseYAxisBound); // default to 0-100% range when doing row-level contribution chart

  if (contributionMode === 'row' && stack) {
    if (min === undefined) min = 0;
    if (max === undefined) max = 1;
  }

  const echartOptions = {
    grid: _extends({}, _defaults.defaultGrid, {
      top: 30,
      bottom: zoomable ? 80 : 0,
      left: 20,
      right: 20
    }),
    xAxis: {
      type: 'time'
    },
    yAxis: _extends({}, _defaults.defaultYAxis, {
      type: logAxis ? 'log' : 'value',
      min,
      max,
      minorTick: {
        show: true
      },
      minorSplitLine: {
        show: minorSplitLine
      },
      axisLabel: {
        formatter: _formatter
      },
      scale: truncateYAxis
    }),
    tooltip: _extends({}, _defaults.defaultTooltip, {
      trigger: 'axis',
      formatter: params => {
        // @ts-ignore
        const rows = ["" + (0, _core.smartDateVerboseFormatter)(params[0].value[0])]; // @ts-ignore

        const prophetValues = (0, _prophet.extractProphetValuesFromTooltipParams)(params);
        Object.keys(prophetValues).forEach(key => {
          const value = prophetValues[key];
          rows.push((0, _prophet.formatProphetTooltipSeries)(_extends({}, value, {
            seriesName: key,
            formatter: _formatter
          })));
        });
        return rows.join('<br />');
      }
    }),
    legend: {
      data: rawSeries.filter(entry => (0, _prophet.extractForecastSeriesContext)(entry.name || '').type === _types.ForecastSeriesEnum.Observation).map(entry => entry.name || ''),
      right: zoomable ? 80 : 'auto'
    },
    series,
    toolbox: {
      show: zoomable,
      feature: {
        dataZoom: {
          yAxisIndex: false,
          title: {
            zoom: 'zoom area',
            back: 'restore zoom'
          }
        }
      }
    },
    dataZoom: zoomable ? [{
      type: 'slider',
      start: 0,
      end: 100,
      bottom: 20
    }] : []
  };
  return {
    area,
    colorScheme,
    contributionMode,
    // @ts-ignore
    echartOptions,
    seriesType,
    logAxis,
    opacity,
    stack,
    markerEnabled,
    markerSize,
    minorSplitLine,
    width,
    height
  };
}